-- ============================================================================
-- == HTConsole - A collection of functions related to interactions that     ==
-- ==             occur via the console. This includes most forms of chat    ==
-- ==             based communication and the display of text-messages       ==
-- ==             within Chat windows.                                       ==
-- ============================================================================

HTConsole = LibStub("AceAddon-3.0"):NewAddon("HTConsole", "AceConsole-3.0", "AceEvent-3.0")
local L   = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTConsole:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTConsole:OnEnable()

  self:RegisterEvent("CHAT_MSG_WHISPER")
  self:RegisterEvent("CHAT_MSG_SYSTEM")

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTConsole:OnDisable()
end

-- ============================================================================
-- == ExecuteChatCommand(arg)                                                ==
-- ==                                                                        ==
-- == This function attempts to execute the supplied argument as though it   ==
-- == were a command entered by the user in the chat window.                 ==
-- ============================================================================

function HTConsole:ExecuteChatCommand(arg)

  if( not DEFAULT_CHAT_FRAME ) then return end

  DEFAULT_CHAT_FRAME.editBox:SetText(arg);
  ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox);

end

-- ============================================================================
-- == SendChatMessage(messageText, channelName, language, recipient)         ==
-- ==                                                                        ==
-- == This function is ostensibly a wrapper to the normal SendChatMessage    ==
-- == function with minor changes as follows:                                ==
-- ==                                                                        ==
-- == 1: Language is checked because it may not be passed in.                ==
-- == 2: If channelName = "CHAT", output is routed to self:Print() instead.  ==
-- == 3: If channelName = "NULL", output is muted.                           ==
-- ==                                                                        ==
-- == The intention for the existence of the NULL option for the channel     ==
-- == name is to allow for functions to pass a parameter that can be         ==
-- == switched between a normal channel and null to turn on/off debug mode   ==
-- == features.                                                              ==
-- ============================================================================

function HTConsole:SendChatMessage(messageText, channelName, language, recipient)

  -- == -----------------------------------------------------------------------
  -- == -- Check whether the language option has been provided.              --
  -- == -----------------------------------------------------------------------
  if language == nil or language == " " then
    language = nil
  end

  channelName = HTConsole:SafeChatChannelName(channelName)

  -- == -----------------------------------------------------------------------
  -- == -- Send chat output to 'chat' or to intended channel if not 'NULL'.  --
  -- == -----------------------------------------------------------------------
  if channelName == "CHAT" then
    HTConsole:PrintChatMessage(messageText)
  elseif channelName == "ERRORFRAME" then
    HTConsole:PrintErrorFrameMessage(messageText)
  elseif channelName ~= "NULL" then
    SendChatMessage(messageText, channelName, language, recipient)
  end

end

-- ============================================================================
-- == PrintChatMessage(messageText)                                          ==
-- ==                                                                        ==
-- == This function prints the supplied message text parameter to the chat   ==
-- == window. Since the chat-window supports colour encoding of content, the ==
-- == text is preprocessed to colourise recognised tokens.                   ==
-- ============================================================================

function HTConsole:PrintChatMessage(messageText)

  -- Colourise: '[HT] - Subtitle - OtherText...', Yellow Subtitle.
  messageText = string.gsub(messageText, "[%-](.-)[%-]", "-|cffffff00%1|r-")

  -- Colourise: '[HT]', Blizzard-Blue [HT].
  messageText = string.gsub(messageText, "[%[]HT[%]]", "|cff00c0ff[HT]|r")

  print(messageText)

end

-- ============================================================================
-- == PrintErrorFrameMessage(messageText)                                    ==
-- ==                                                                        ==
-- == This function prints the supplied message text parameter to the UI     ==
-- == error-frame.                                                           ==
-- ============================================================================

function HTConsole:PrintErrorFrameMessage(messageText)

  UIErrorsFrame:AddMessage(messageText, 0, 1.0, 1.0, 1.0, 1)
  
end

-- ============================================================================
-- == SendWrappedMessage(messageText, channelName)                           ==
-- ==                                                                        ==
-- == This function processes the supplied messageText data into a series of ==
-- == individual messages which are then output to the channel identified by ==
-- == the channelName parameter.                                             ==
-- ============================================================================

function HTConsole:SendWrappedMessage(messageText, channelName, language, recipient)

  local maxWidth        = HTConfig:GetChatWindowTextMaxWidth(nil)
  local words           = HTData:StringSplit(messageText, " ")
  local characterCount  = 0
  local thisLineText    = " "
  local spacerChar      = ""

  HTConsole:Debug("maxWidth=[" .. maxWidth .. "]")

  -- Loop through the list of words, printing as many per line as the maximum
  -- line width allows.
  for i,v in ipairs(words) do

    HTConsole:Debug("Line: [" .. thisLineText .. "], Word: [" .. v .. "], Length: [" .. characterCount .. "], Limit: [" .. maxWidth .. "]")
    if (characterCount + string.len(v)) < maxWidth then
      thisLineText   = thisLineText .. spacerChar .. v
      characterCount = characterCount + string.len(v) + 1
      spacerChar     = " "
    else
      SendChatMessage(thisLineText, channelName, language, recipient)
      thisLineText      = spacerChar .. v .. spacerChar
      characterCount    = 0
      spacerChar        = ""
    end

  end

  -- Check if there are any words remaining to be printed as a final non-full
  -- line.
  if string.len(thisLineText) > 0 then
    SendChatMessage(thisLineText, channelName, language, recipient)
  end

end

-- ============================================================================
-- == SendTitleMessage(messageText, channelName)                             ==
-- ==                                                                        ==
-- == This function processes the supplied messageText data into a series of ==
-- == individual messages which are then output to the channel identified by ==
-- == the channelName parameter.                                             ==
-- ============================================================================

function HTConsole:SendTitleMessage(messageText, channelName)


end

-- ============================================================================
-- == SendTitleBar(channelName)                                              ==
-- ==                                                                        ==
-- == This function outputs a title-bar configured to the width specified in ==
-- == the configuration property describing the maximum chat window width.   ==
-- ============================================================================

function HTConsole:SendTitleBar(channelName)

  local titleBarText = ""
  local maxWidth     = abs(HTConfig:GetChatWindowTextMaxWidth(nil) * 1.5 )

  for i=1,maxWidth do
    titleBarText = titleBarText .. "-"
  end

  HTConsole:SendChatMessage(HTConsole:GetTitleBar(), channelName, nil, nil)

end

-- ============================================================================
-- == GetTitleBar()                                                          ==
-- ==                                                                        ==
-- == This function returns a title-bar whose width is defined by the        ==
-- == current max-width calculation.                                         ==
-- ============================================================================

function HTConsole:GetTitleBar()

  local titleBarText = ""
  local maxWidth     = abs(HTConfig:GetChatWindowTextMaxWidth(nil) * 1.5 )

  for i=1,maxWidth do
    titleBarText = titleBarText .. "-"
  end

  return titleBarText

end

-- ============================================================================
-- == Debug(messageText)                                                     ==
-- ==                                                                        ==
-- == This function outputs the supplied message text to the "CHAT" channel  ==
-- == as a standard format 'debug' message if the debug-mode is currently    ==
-- == enabled.                                                               ==
-- ============================================================================

function HTConsole:Debug(messageText)

  if messageText == nil then return end

  if HTConfig:IsFeatureEnabled(L["cfg-feature-name-debugmode"]) then

    local messageText = "|cffff0000" .. "[DEBUG]" .. "|r" .. " - " .. messageText

    HTConsole:SendChatMessage(messageText, "CHAT", nil, nil)

  end

end

-- ============================================================================
-- == SafeChatChannelName(channelName)                                       ==
-- ==                                                                        ==
-- == This function simply checks that the supplied channelName parameter    ==
-- == would be safe to use in the context of this user. If the name would be ==
-- == considered unsafe, a safe alternative is substituted.                  ==
-- ============================================================================

function HTConsole:SafeChatChannelName(channelName)

  -- == -----------------------------------------------------------------------
  -- == -- Setup variables that will be needed.                              --
  -- == -----------------------------------------------------------------------
  --  local playerName = UnitName("player")

  -- == -----------------------------------------------------------------------
  -- == -- [GUILD] Checks...                                                 --
  -- == -----------------------------------------------------------------------
  if channelName == "GUILD" then
     if not IsInGuild() then
        channelName = "CHAT"
     end
  end

  -- == -----------------------------------------------------------------------
  -- == -- [RAID] Checks...                                                  --
  -- == -----------------------------------------------------------------------
  if channelName == "RAID" then
     if UnitInRaid(playerName) == -1 then
        channelName = "PARTY"
        print("channelName set to Party")
     end
  end

  -- == -----------------------------------------------------------------------
  -- == -- [PARTY] Checks...                                                 --
  -- == -----------------------------------------------------------------------
  if channelName == "PARTY" then
     if UnitInParty(playerName) == -1 then
        channelName = "CHAT"
        print("channelName set to Chat")
     end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Return the value, adjusted or otherwise, of channelName.          --
  -- == -----------------------------------------------------------------------
  return channelName

end

-- ============================================================================
-- == CHAT_MSG_WHISPER(channel, msg, sender)                                 ==
-- ==                                                                        ==
-- == This function performs actions when the CHAT_MSG_WHISPER event is      ==
-- == triggered. It is triggered when the player's character receives a      ==
-- == message via the whisper 'channel.'                                     ==
-- ============================================================================

function HTConsole:CHAT_MSG_WHISPER(channel, msg, sender)

  HTConsole:DoAutoInvitations(msg, sender)
  HTConsole:DoAutoInfoResponse(msg, sender)

end

-- ============================================================================
-- == CHAT_MSG_SYSTEM(msg)                                                   ==
-- ==                                                                        ==
-- == This function performs actions when the CHAT_MSG_SYSTEM event is       ==
-- == triggered. It is triggered when the player's character receives a      ==
-- == message via the system 'channel.'                                      ==
-- ==                                                                        ==
-- ============================================================================

function HTConsole:CHAT_MSG_SYSTEM(msg)

  HTConsole:DoAutoRaidPromotions(msg)

end

-- ============================================================================
-- == DoAutoInvitations(msg, sender)                                         ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text for a recognised request  ==
-- == for a group invitation and sends one automatically when one is found.  ==
-- ============================================================================

function HTConsole:DoAutoInvitations(msg, sender)

  -- == -----------------------------------------------------------------------
  -- == -- Check if the sender was requesting an invite.                     --
  -- == -----------------------------------------------------------------------
  if HTConsole:IsInvitationKeyword(string.lower(msg)) and HTConfig:IsFeatureEnabled(L["cfg-feature-name-autoinvitations"]) then
    HTConsole:SendChatMessage(L["con-autoinvitations-msg-01"]  , "WHISPER", nil, sender)
    HTConsole:ExecuteChatCommand(L["con-autoinvitations-cmd-01"]..sender)
  end

end

-- ============================================================================
-- == IsInvitationKeyword(msg)                                               ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text to determine if it        ==
-- == matches one of the recognised invitation keywords.                     ==
-- ============================================================================

function HTConsole:IsInvitationKeyword(msg)

  if msg == L["con-autoinvitations-keyword-01"]
  or msg == L["con-autoinvitations-keyword-02"] then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == DoAutoRaidPromotions(msg)                                              ==
-- ==                                                                        ==
-- == This function will attempt to automatically promote members joining a  ==
-- == raid group to assistant status if certain criteria are met:            ==
-- ==                                                                        ==
-- == - The auto-promotions feature is enabled.                              ==
-- == - This player is the raid leader.                                      ==
-- == - The player joining the raid has an officer-class guild rank.         ==
-- ============================================================================

function HTConsole:DoAutoRaidPromotions(msg)

  -- Check feature is enabled and player is raid leader, return if either is false.
  if not HTConfig:IsFeatureEnabled(L["cfg-feature-name-autoraidpromotions"]) or not IsRaidLeader() then return end

  -- If the system message was of the form "{playername} has joined the raid group"...
  if string.match(msg, L["con-autoraidpromotions-system-msg-01"]) then
    local words         = HTData:StringSplit(msg, " ")
    local newRaidMember = HTData:SafeString(words[1])

    -- AutoPromote players with an officer class guild rank.
    if HTCommon:IsOfficer(newRaidMember) then
      HTConsole:Debug("Attempting to promote "..newRaidMember.." to assistant status.")
      HTConsole:SendChatMessage(L["con-autoraidpromotions-msg-01"], "WHISPER", nil, newRaidMember)
      PromoteToAssistant(newRaidMember)
    end
  end

end

-- ============================================================================
-- == DoAllRaidPromotions()                                                  ==
-- ==                                                                        ==
-- == This function will attempt to automatically promote any members of a   ==
-- == raid group to assistant status if certain criteria are met:            ==
-- ==                                                                        ==
-- == - This player is the raid leader.                                      ==
-- == - The raid member holds an officer-class guild rank.                   ==
-- ============================================================================

function HTConsole:DoAllRaidPromotions()

  -- Check player is raid leader, otherwise return.
  if not IsRaidLeader() then return end

  -- Iterate through members of current raid.
  for i = 1, GetNumRaidMembers() do
    local name, rank = GetRaidRosterInfo(i)
    local raidMember = HTData:SafeString(name)
    local raidRank   = HTData:SafeNumber(rank)

    -- Only process players that are not already promoted to raid assistant.
    if raidRank > 2 then

      -- Only process 'other' players. IE: Not this user.
      if not HTCommon:IsMe(raidMember) then

        -- AutoPromote players with an officer class guild rank.
        if HTCommon:IsOfficer(raidMember) then
          HTConsole:Debug("Attempting to promote "..raidMember.." to assistant status.")
          HTConsole:SendChatMessage(L["con-autoraidpromotions-msg-01"], "WHISPER", nil, raidMember)
          PromoteToAssistant(raidMember)
        end
      end
    end
  end

end

-- ============================================================================
-- == DoAutoInfoResponse(msg, sender)                                        ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text for a recognised request  ==
-- == for information and sends a suitable response where appropriate.       ==
-- ============================================================================

function HTConsole:DoAutoInfoResponse(msg, sender)

  -- == -----------------------------------------------------------------------
  -- == -- Check if the sender was requesting an invite.                     --
  -- == -----------------------------------------------------------------------
  if HTConsole:IsInfoKeyword(string.lower(msg)) then

    -- Info Request = "info"...
    if     string.lower(msg) == L["con-autoinforesponse-keyword-info"] then
      HTConsole:SendAutoInfoResponse(sender)

    -- Info Request = "info contactus"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-contactus"] then
      HTConsole:SendAutoInfoContactUsResponse(sender)

    -- Info Request = "info guild"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-guild"] then
      HTConsole:SendAutoInfoGuildResponse(sender)

    -- Info Request = "info officerlist"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-officerlist"] then
      HTConsole:SendAutoInfoOfficerListResponse(sender)

    -- Info Request = "info raidtimes"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-raidtimes"] then
      HTConsole:SendAutoInfoRaidTimesResponse(sender)

    -- Info Request = "info recruitment"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-recruitment"] then
      HTConsole:SendAutoInfoRecruitmentResponse(sender)

    -- Info Request = "info zat"...
    elseif string.lower(msg) == L["con-autoinforesponse-keyword-zat"] then
      HTConsole:SendAutoInfoZATResponse(sender)

    -- Info Request = "no match to any of the above"...
    else
      HTConsole:SendChatMessage(L["con-autoinforesponse-msg-01"]  , "WHISPER", nil, sender)
    end

  end

end

-- ============================================================================
-- == IsInfoKeyword(msg)                                                     ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text to determine if it        ==
-- == matches one of the recognised info keywords.                           ==
-- ============================================================================

function HTConsole:IsInfoKeyword(msg)

  if msg == L["con-autoinforesponse-keyword-info"]
  or msg == L["con-autoinforesponse-keyword-contactus"]
  or msg == L["con-autoinforesponse-keyword-guild"]
  or msg == L["con-autoinforesponse-keyword-officerlist"]
  or msg == L["con-autoinforesponse-keyword-raidtimes"]
  or msg == L["con-autoinforesponse-keyword-recruitment"]
  or msg == L["con-autoinforesponse-keyword-zat"] then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == SendInfoMessage(infoTitle, messageText, sender)                        ==
-- ==                                                                        ==
-- == This function sends the messageText to the sender using the infoTitle  ==
-- == provided.                                                              ==
-- ============================================================================

function HTConsole:SendInfoMessage(infoTitle, messageText, sender)
  
  HTConsole:SendChatMessage(HTConsole:GetTitleBar(), "WHISPER", nil, sender)
  HTConsole:SendChatMessage(L["con-autoinforesponse-title-msg-01"] .. infoTitle, "WHISPER", nil, sender)
  HTConsole:SendChatMessage(HTConsole:GetTitleBar(), "WHISPER", nil, sender)
  HTConsole:SendChatMessage(" ", "WHISPER", nil, sender)
  HTConsole:SendWrappedMessage(messageText, "WHISPER", nil, sender)
  HTConsole:SendChatMessage(" ", "WHISPER", nil, sender)
  HTConsole:SendChatMessage(HTConsole:GetTitleBar(), "WHISPER", nil, sender)  
  HTConsole:SendChatMessage(L["con-autoinforesponse-title-msg-01"] .. L["con-autoinforesponse-topics-msg-01"], "WHISPER", nil, sender)
  HTConsole:SendChatMessage(HTConsole:GetTitleBar(), "WHISPER", nil, sender)

end

-- ============================================================================
-- == SendAutoInfoResponse(sender)                                           ==
-- ==                                                                        ==
-- == This function sends the main topic list info to the sender.            ==
-- ============================================================================

function HTConsole:SendAutoInfoResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-info-msg-01"], L["con-autoinforesponse-info-msg-02"], sender)

end

-- ============================================================================
-- == SendAutoInfoGuildResponse(sender)                                      ==
-- ==                                                                        ==
-- == This function sends the general guild info to the sender.              ==
-- ============================================================================

function HTConsole:SendAutoInfoGuildResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-guild-msg-01"], L["con-autoinforesponse-guild-msg-02"], sender)

end

-- ============================================================================
-- == SendAutoInfoOfficerListResponse(sender)                                ==
-- ==                                                                        ==
-- == This function sends officer list info to the sender.                   ==
-- ============================================================================

function HTConsole:SendAutoInfoOfficerListResponse(sender)

  local messageContent   = L["con-autoinforesponse-officerlist-msg-02"]
  local guildOfficerList = HTCommon:GetGuildOfficerList()

  for i,v in ipairs(guildOfficerList) do

    if i > 1 then
      messageContent = messageContent .. ", "
    end

    messageContent = messageContent .. guildOfficerList[i]

  end

  HTConsole:SendInfoMessage(L["con-autoinforesponse-officerlist-msg-01"], messageContent, sender)

end

-- ============================================================================
-- == SendAutoInfoRaidTimesResponse(sender)                                  ==
-- ==                                                                        ==
-- == This function sends raid times info to the sender.                     ==
-- ============================================================================

function HTConsole:SendAutoInfoRaidTimesResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-raidtimes-msg-01"], L["con-autoinforesponse-raidtimes-msg-02"], sender)

end

-- ============================================================================
-- == SendAutoInfoRecruitmentResponse(sender)                                ==
-- ==                                                                        ==
-- == This function sends the recruitment info to the sender.                ==
-- ============================================================================

function HTConsole:SendAutoInfoRecruitmentResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-recruitment-msg-01"], L["con-autoinforesponse-recruitment-msg-02"], sender)

end

-- ============================================================================
-- == SendAutoInfoContactUsResponse(sender)                                  ==
-- ==                                                                        ==
-- == This function sends the contact us info to the sender.                 ==
-- ============================================================================

function HTConsole:SendAutoInfoContactUsResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-contactus-msg-01"], L["con-autoinforesponse-contactus-msg-02"], sender)

end

-- ============================================================================
-- == SendAutoInfoZATResponse(sender)                                        ==
-- ==                                                                        ==
-- == This function sends the ZAT info to the sender.                        ==
-- ============================================================================

function HTConsole:SendAutoInfoZATResponse(sender)

  HTConsole:SendInfoMessage(L["con-autoinforesponse-zat-msg-01"], L["con-autoinforesponse-zat-msg-02"], sender)

end
